@@ -268,4 +268,47 @@ public interface NinaAction {
 	 */
 	public NinaAction empty(NinaEvent q, String name);
 
+	/**
+	 * 
+	 * @param q
+	 * @param digits
+	 * @param sign
+	 */
+	public void setEdgeDigits(NinaEvent q, int digits, boolean sign);
+
+	/**
+	 * 
+	 * @param q
+	 * @param bits
+	 */
+	public void setEdgeHexadecimal(NinaEvent q, int bits);
+
+	/**
+	 * 
+	 * @param q
+	 * @param bits
+	 */
+	public void setEdgeOctal(NinaEvent q, int bits);
+
+	/**
+	 * 
+	 * @param q
+	 */
+	public void setEdgeFloat(NinaEvent q);
+
+	/**
+	 * 
+	 * @param q
+	 * @param cmp
+	 * @param off
+	 */
+	public void setEdgeIndent(NinaEvent q, String cmp, int off);
+
+	/**
+	 * 
+	 * @param q
+	 * @param lang
+	 */
+	public void setEdgeIdentifier(NinaEvent q, String lang);
+
 }
