@@ -1338,6 +1338,32 @@ public class Quadro implements NinaEvent {
 		}
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public int getX() {
+		return curx;
+	}
+
+	/**
+	 * 
+	 * @return
+	 */
+	public int getY() {
+		return cury;
+	}
+
+	/**
+	 * 
+	 * @param x
+	 * @param y
+	 */
+	public void setPosition(int x, int y) {
+		curx = x;
+		cury = y;
+	}
+
 	/* (non-Javadoc)
 	 * @see java.lang.Object#toString()
 	 */
