@@ -241,9 +241,9 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 				b = ((DFABuilder.DBS)dfa).getUserEdge(x);
 				o = ((DFABuilder.DBS)dfa).getUserMealyEdge(x);
 				if(els) {
-					out.printf("\t\t\t} else if(__g && (%s)) {\n", x);
+					out.printf("\t\t\t} else if(%s) {\n", x);
 				} else {
-					out.printf("\t\t\tif(__g && (%s)) {\n", x);
+					out.printf("\t\t\tif(%s) {\n", x);
 				}
 
 				if(o != null) {
@@ -380,6 +380,16 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\t\t__this__.STATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\t\treturn 1;");
+			return;
+		}
+
 		for(Interval v : dfa.getAlphabetRanges()) {
 			c = ((Integer)v.getInfimumBound()).intValue();
 			if(v.isInfimumOpen())  c++;
@@ -412,6 +422,16 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\t\t__this__.STATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\t\treturn 1;");
+			return;
+		}
+
 		for(Object p : dfa.getAlphabets()) {
 			printobj(out, p, els, o = dfa.getLabel(p));
 			printIntervalAction(out, dfa, o);
@@ -442,6 +462,16 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\t\t__this__.STATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\t\treturn 1;");
+			return;
+		}
+
 		for(Object p : dfa.getAlphabets()) {
 			if(!printclass1(out, p, els, o = dfa.getLabel(p))) {
 				continue;
@@ -571,17 +601,14 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
-	 */
-	@Override
-	public void printAcceptStates(PrintStream out) {
+	//
+	private void printPStates(PrintStream out, Set<Integer> a) {
 		String d = "\t\treturn (";
 
-		if(acceptsSize() == 0) {
+		if(a.size() == 0) {
 			out.println("\t\treturn false;");
 		} else {
-			for(Integer i : acceptsIterable()) {
+			for(Integer i : a) {
 				out.print(d);
 				out.format("__this__.STATE == %d", i);
 				d = " ||\n\t\t\t\t";
@@ -590,6 +617,30 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printAcceptStates(PrintStream out) {
+		printPStates(out, acceptsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printDeadStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printDeadStates(PrintStream out) {
+		printPStates(out, deadsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printEmptyTransitionStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printEmptyTransitionStates(PrintStream out) {
+		printPStates(out, emptyTransitionsIterable());
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptToken(java.io.PrintStream)
 	 */
@@ -667,7 +718,10 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 			} else {
 				p = ReplaceAction.replace(p, a, this, stateNo(s),
 						builder.getLabelByState(s));
-				out.format("\t\t\t%s\n", p);
+				for(String v : p.split("\n")) {
+					out.format("\t\t\t%s\n", v);
+				}
+
 				if(!a[0]) {
 					out.println("\t\t\tbreak;");
 				}
@@ -814,11 +868,26 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#openScript()
 	 */
 	@Override
-	protected InputStream openScript() {
-		return NinaTranslator.class.getResourceAsStream(
-				"/net/morilib/nina/translate/nina_template." +
-				getMachine() +
-				".cs.sh");
+	protected InputStream openScript(ShEnvironment env) {
+		if(options.getInjectFile() == null) {
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_template." +
+					getMachine() +
+					".cs.sh");
+		} else {
+			env.bind("TRANS_COMMAND",
+					"nina_template." + getMachine() + ".cs.sh");
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_injector.sh");
+		}
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.NinaTranslator#getExtension()
+	 */
+	@Override
+	public String getExtension() {
+		return ".cs";
 	}
 
 	/* (non-Javadoc)
@@ -827,7 +896,7 @@ public class NinaTranslatorCSharp extends AbstractNinaTranslator {
 	@Override
 	protected PrintStream openOutput() throws IOException {
 		return new PrintStream(getOptions().getOutputStream(
-				getOptions().getTitleFile(".cs")), true);
+				getOptions().getTitleFile(getExtension())), true);
 	}
 
 	/* (non-Javadoc)
