@@ -209,7 +209,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	}
 
 	private boolean putend(String s, PrintStream out,
-			DFAState<Object, ?, Void> dfa, boolean els) {
+			DFAState<Object, ?, Void> dfa, boolean els, int[] all) {
 		DFABuilder.DBS b;
 		TMBuilder.DBS x;
 		Object o;
@@ -223,6 +223,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(b));
 			out.format("\t\t\t\treturn 1;\n");
 			els = true;
+			all[0]++;
 		} else if(dfa instanceof TMBuilder.DBS &&
 				(x = ((TMBuilder.DBS)dfa).getEnd()) != null) {
 			outln(els, out, s);
@@ -231,6 +232,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(x));
 			out.format("\t\t\t\treturn 1;\n");
 			els = true;
+			all[0]++;
 		}
 		return els;
 	}
@@ -264,7 +266,8 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	}
 
 	private boolean putothers(PrintStream out,
-			DFAState<Object, ?, Void> dfa, boolean els, String so) {
+			DFAState<Object, ?, Void> dfa, boolean els, String so,
+			int[] all) {
 		DFABuilder.DBS a, b;
 		TMBuilder.DBS x;
 		String s, t;
@@ -277,6 +280,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 						((TMBuilder.DBS)dfa).getMealyOthers());
 				out.format("%s\t\t\tSTATE = %d;\n", s, getStateNo(x));
 				out.format("%s\t\t\treturn 1;\n", s);
+				all[0]++;
 				return true;
 			} else {
 				return els;
@@ -290,6 +294,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			}
 			out.format("%s\t\t\tSTATE = %d;\n", s, getStateNo(b));
 			out.format("%s\t\t\treturn 1;\n", s);
+			all[0]++;
 			return true;
 		} else if((b = a.getRecursive()) == null) {
 			return els;
@@ -300,6 +305,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 					getStateNo(b), t);
 			out.format("%s\t\t\tSTATE = 0;\n", s);
 			out.format("%s\t\t\treturn NINA_ACCEPT;\n", s);
+			all[0]++;
 			return true;
 		} else {
 			s = outln(els, out, so);
@@ -308,6 +314,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 					getStateNo(b), t);
 			out.format("%s\t\t\tSTATE = 0;\n", s);
 			out.format("%s\t\t\treturn NINA_ACCEPT;\n", s);
+			all[0]++;
 			return true;
 		}
 	}
@@ -409,11 +416,19 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			DFAState<Object, ?, Void> dfa) {
 		Map<Interval, Object> r = new HashMap<Interval, Object>();
 		int sn = getStateNo(dfa), c;
+		int[] a = new int[1];
 		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
 
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			printNextStep(out, ((DFABuilder.DBS)dfa).getOthers());
+			return;
+		}
+
 		// with lookahead
 		for(Interval v : dfa.getAlphabetRanges()) {
 			if(v.isEmpty())  continue;
@@ -445,11 +460,12 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		// dynamic automaton
 		els = printDynamicTransition(out, dfa, els);
 
-		els = putend("$c < 0", out, dfa, els);
+		a[0] = 0;
+		els = putend("$c < 0", out, dfa, els, a);
 		els = putuseredges(out, dfa, els);
-		els = putothers(out, dfa, els, "$c >= 0");
+		els = putothers(out, dfa, els, "$c >= 0", a);
 		if(els)  out.println("\t\t\t}");
-		out.println("\t\t\treturn 0;");
+		if(a[0] < 2)  out.println("\t\t\treturn 0;");
 	}
 
 	//
@@ -457,11 +473,19 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			DFAState<Object, ?, Void> dfa) {
 		Map<Object, Object> r = new HashMap<Object, Object>();
 		int sn = getStateNo(dfa);
+		int[] a = new int[1];
 		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
 
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			printNextStep(out, ((DFABuilder.DBS)dfa).getOthers());
+			return;
+		}
+
 		// with lookahead
 		for(Object p : dfa.getAlphabets()) {
 			if(!isLookahead(o = dfa.getLabel(p)))  continue;
@@ -487,11 +511,12 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		// dynamic automaton
 		els = printDynamicTransition(out, dfa, els);
 
-		els = putend("$c == null", out, dfa, els);
+		a[0] = 0;
+		els = putend("$c == null", out, dfa, els, a);
 		els = putuseredges(out, dfa, els);
-		els = putothers(out, dfa, els, "$c != null");
+		els = putothers(out, dfa, els, "$c != null", a);
 		if(els)  out.println("\t\t\t}");
-		out.println("\t\t\treturn 0;");
+		if(a[0] < 2)  out.println("\t\t\treturn 0;");
 	}
 
 	//
@@ -499,11 +524,19 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			DFAState<Object, ?, Void> dfa) {
 		Map<Object, Object> r = new HashMap<Object, Object>();
 		int sn = getStateNo(dfa);
+		int[] a = new int[1];
 		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
 
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			printNextStep(out, ((DFABuilder.DBS)dfa).getOthers());
+			return;
+		}
+
 		// with lookahead pass 1
 		for(Object p : dfa.getAlphabets()) {
 			if(!isLookahead(o = dfa.getLabel(p)))  continue;
@@ -555,11 +588,12 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		// dynamic automaton
 		els = printDynamicTransition(out, dfa, els);
 
-		els = putend("$c == null", out, dfa, els);
+		a[0] = 0;
+		els = putend("$c == null", out, dfa, els, a);
 		els = putuseredges(out, dfa, els);
-		els = putothers(out, dfa, els, "$c != null");
+		els = putothers(out, dfa, els, "$c != null", a);
 		if(els)  out.println("\t\t\t}");
-		out.println("\t\t\treturn 0;");
+		if(a[0] < 2)  out.println("\t\t\treturn 0;");
 	}
 
 	//
@@ -655,17 +689,14 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
-	 */
-	@Override
-	public void printAcceptStates(PrintStream out) {
+	//
+	private void printPStates(PrintStream out, Set<Integer> a) {
 		String d = "\t\treturn (";
 
-		if(acceptsSize() == 0) {
+		if(a.size() == 0) {
 			out.println("\t\treturn false;");
 		} else {
-			for(Integer i : acceptsIterable()) {
+			for(Integer i : a) {
 				out.print(d);
 				out.format("STATE == %d", i);
 				d = " ||\n\t\t\t\t";
@@ -674,6 +705,30 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printAcceptStates(PrintStream out) {
+		printPStates(out, acceptsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printDeadStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printDeadStates(PrintStream out) {
+		printPStates(out, deadsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printEmptyTransitionStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printEmptyTransitionStates(PrintStream out) {
+		printPStates(out, emptyTransitionsIterable());
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptToken(java.io.PrintStream)
 	 */
@@ -893,11 +948,26 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#openScript()
 	 */
 	@Override
-	protected InputStream openScript() {
-		return NinaTranslator.class.getResourceAsStream(
-				"/net/morilib/nina/translate/nina_template." +
-				getMachine() +
-				".java.sh");
+	protected InputStream openScript(ShEnvironment env) {
+		if(options.getInjectFile() == null) {
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_template." +
+					getMachine() +
+					".java.sh");
+		} else {
+			env.bind("TRANS_COMMAND",
+					"nina_template." + getMachine() + ".java.sh");
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_injector.sh");
+		}
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.NinaTranslator#getExtension()
+	 */
+	@Override
+	public String getExtension() {
+		return ".java";
 	}
 
 	/* (non-Javadoc)
@@ -906,7 +976,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	@Override
 	protected PrintStream openOutput() throws IOException {
 		return new PrintStream(getOptions().getOutputStream(
-				getOptions().getTitleFile(".java")), true);
+				getOptions().getTitleFile(getExtension())), true);
 	}
 
 	/* (non-Javadoc)
