@@ -360,6 +360,16 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\tSTATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\treturn 1;");
+			return;
+		}
+
 		for(Interval v : dfa.getAlphabetRanges()) {
 			if(v.isEmpty())  continue;
 			c = infimumbound(v);
@@ -393,6 +403,16 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\tSTATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\treturn 1;");
+			return;
+		}
+
 		for(Object p : dfa.getAlphabets()) {
 			printobj(out, p, els, o = dfa.getLabel(p));
 			printIntervalAction(out, dfa, o);
@@ -423,6 +443,16 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// if the state has only empty transition
+		if(dfa instanceof DFABuilder.DBS &&
+				((DFABuilder.DBS)dfa).isOnlyEmptyTransition()) {
+			out.format("\t\tSTATE = %d;\n",
+					getStateNo(((DFABuilder.DBS)dfa).getOthers()));
+			out.println("\t\treturn 1;");
+			return;
+		}
+
 		for(Object p : dfa.getAlphabets()) {
 			if(!printclass1(out, p, els, o = dfa.getLabel(p))) {
 				continue;
@@ -552,17 +582,14 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
-	 */
-	@Override
-	public void printAcceptStates(PrintStream out) {
+	//
+	private void printPStates(PrintStream out, Set<Integer> a) {
 		String d = "\treturn (";
 
-		if(acceptsSize() == 0) {
+		if(a.size() == 0) {
 			out.println("\treturn false;");
 		} else {
-			for(Integer i : acceptsIterable()) {
+			for(Integer i : a) {
 				out.print(d);
 				out.format("STATE == %d", i);
 				d = " ||\n\t\t\t";
@@ -571,6 +598,30 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printAcceptStates(PrintStream out) {
+		printPStates(out, acceptsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printDeadStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printDeadStates(PrintStream out) {
+		printPStates(out, deadsIterable());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printEmptyTransitionStates(java.io.PrintStream)
+	 */
+	@Override
+	public void printEmptyTransitionStates(PrintStream out) {
+		printPStates(out, emptyTransitionsIterable());
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printAcceptToken(java.io.PrintStream)
 	 */
@@ -789,10 +840,25 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#openScript()
 	 */
 	@Override
-	protected InputStream openScript() {
-		return NinaTranslator.class.getResourceAsStream(
-				"/net/morilib/nina/translate/nina_template." +
-				getMachine() + ".js.sh");
+	protected InputStream openScript(ShEnvironment env) {
+		if(options.getInjectFile() == null) {
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_template." +
+					getMachine() + ".js.sh");
+		} else {
+			env.bind("TRANS_COMMAND",
+					"nina_template." + getMachine() + ".js.sh");
+			return NinaTranslator.class.getResourceAsStream(
+					"/net/morilib/nina/translate/nina_injector.sh");
+		}
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.NinaTranslator#getExtension()
+	 */
+	@Override
+	public String getExtension() {
+		return ".js";
 	}
 
 	/* (non-Javadoc)
@@ -802,7 +868,8 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 	protected PrintStream openOutput() throws IOException {
 		return new PrintStream(getOptions().getOutputStream(
 				getOptions().getOutputDir(),
-				getOptions().getOutputFilename() + ".js"), true);
+				getOptions().getOutputFilename() + getExtension()),
+				true);
 	}
 
 	/* (non-Javadoc)
