@@ -18,12 +18,15 @@ putlicense_cstyle.sh
 buffer='$buffer'
 int='$int'
 num='$num'
+indent='$indent'
+indentBefore='$indentBefore'
 
 echo '/**'
 awk '{ printf(" * %s\n", $0) }' description
 echo ' */'
 cat definition
 
+echo '/* @@@-PARSER-CODE-START-@@@ */'
 echo "function ${CLASSNAME}() {"
 cat << EOF
 	this.__state = 0;
@@ -50,6 +53,7 @@ cat << EOF
 	this.$num = null;
 
 EOF
+
 [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
 	this.__backtrack = null;
 	this.__backtrack_ptr = -1;
@@ -63,6 +67,7 @@ EOF
 	this.__backtrack_sptr = -1;
 	this.__backtrack_flg = false;
 EOF
+
 [ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
 	this.__lookahead_state = 0;
 	this.__lookahead_mark = -1;
@@ -72,6 +77,14 @@ EOF
 	this.__lookaheadw_ptr = -1;
 	this.__lookahead_ok = true;
 EOF
+
+[ -n "$ENABLE_INDENT" ] && cat << EOF
+	this.$indent = 0;
+	this.$indentBefore = 0;
+	this.__is_line_begin = true;
+	this.__indent_unread = null;
+EOF
+
 for i in $SUBAUTOMATA
 do
   ch1=`replace_strange_char $i`
@@ -110,11 +123,38 @@ ${CLASSNAME}.__log = null;
 EOF
 
 function _putreadf {
-  cat << EOF
+  if [ -n "$ENABLE_INDENT" ]; then
+    cat << EOF
+${CLASSNAME}.prototype.${1} = function(rd) {
+	var c;
+
+	if(this.__indent_unread != null) {
+		c = this.__indent_unread;
+		this.__indent_unread = null;
+	} else if(!this.__is_line_begin) {
+		c = rd();
+	} else {
+		this.$indent = 0;
+		while((c = rd()) == ' '.charCodeAt(0) || c == '\t'.charCodeAt(0)) {
+			this.$indent++;
+		}
+		this.__indent_unread = c;
+		c = c != null ? INDENT : c;
+	}
+
+	if(this.__is_line_begin = c == '\n'.charCodeAt(0)) {
+		this.$indentBefore = this.$indent;
+	}
+	return c;
+}
+EOF
+  else
+    cat << EOF
 ${CLASSNAME}.prototype.${1} = function(rd) {
 	return rd();
 }
 EOF
+  fi
 }
 
 cat field
@@ -181,6 +221,7 @@ fi
 cat << EOF
 
 ${CLASSNAME}.__SKIP__ = {};
+${CLASSNAME}.__DUM__  = {};
 
 ${CLASSNAME}.prototype._read = function(rd) {
 	var c;
@@ -472,6 +513,18 @@ EOF
   cat << EOF
 }
 
+${CLASSNAME}.ENGINE_${ch1}.dead = function($this) {
+EOF
+  print_deads $i
+  cat << EOF
+}
+
+${CLASSNAME}.ENGINE_${ch1}.emptyTransition = function($this) {
+EOF
+  print_empty_transitions $i
+  cat << EOF
+}
+
 ${CLASSNAME}.ENGINE_${ch1}.toString = function() {
 	return "${ch1}";
 }
@@ -516,6 +569,14 @@ DynamicEngine.prototype.accepted = function($this) {
 	return true;
 }
 
+DynamicEngine.prototype.dead = function($this) {
+	return false;
+}
+
+DynamicEngine.prototype.emptyTransition = function($this) {
+	return false;
+}
+
 DynamicEngine.prototype.execaction = function($this, c) {
 	return 1;
 }
@@ -586,7 +647,7 @@ ${CLASSNAME}.prototype._parse = function(rd, x, rt, st) {
 		do {
 			en = this.__stk[this.__slen - 1];
 			if(c === ${CLASSNAME}.__SKIP__) {
-				// do nothing
+				c = ${CLASSNAME}.__DUM__;
 			} else if((a = en.step(this, rd, c)) > 0) {
 				this.__logprint("transit to state " + this.__state + ": ", c);
 				b = en.accepted(this);
@@ -647,7 +708,34 @@ EOF
 				this._f_UNGET(c);
 				st[0] = NINA_FAIL;  return null;
 			}
-			c = this._read(rd);
+
+			if(this.__stk[this.__slen - 1].emptyTransition(this)) {
+				// do nothing
+			} else if(!this.__stk[this.__slen - 1].dead(this)) {
+				c = this._read(rd);
+			} else if(b) {
+				this.__logprint("accept " + this.__stk[this.__slen - 1].toString());
+				st[0] = NINA_ACCEPT;  return null;
+EOF
+  [ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
+			} else if(this.__lookaheadw_ptr >= 0) {
+				this.__logprint("match failed: try lookahead: ", c);
+				this._f_LOOKAHEAD_RB();
+				b = en.accepted(this);
+EOF
+  [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
+			} else if(this._f_BACKTRACK(c)) {
+				this.__logprint("match failed: try backtracking: ", c);
+EOF
+  cat << EOF
+			} else if(c == null) {
+				if(!b)  throw "${TOKENERROR}";
+				st[0] = NINA_ACCEPT;  return null;
+			} else {
+				this.__logprint("match failed: ", c);
+				this.__puttrace();
+				st[0] = NINA_FAIL;  return null;
+			}
 		} while(true);
 	} catch(e) {
 		this._f_UNGET(c);
@@ -800,6 +888,18 @@ for i in $SUBAUTOMATA
 do
   print_constants $i
 done
+echo '/* @@@-PARSER-CODE-END-@@@ */'
 
 cat fragment
 #cat fragment | replace_action
+
+[ -n "$PUTMAIN" ] && cat << EOF
+${CLASSNAME}.parseStdin();
+EOF
+[ -n "$PUTPROMPT" ] && cat << EOF
+var s;
+
+while((s = java.lang.System.console().readLine("> ")) != null) {
+	${CLASSNAME}.parseStream(new java.io.StringReader(s));
+}
+EOF
